@@ -6,7 +6,6 @@ from __future__ import division
 import time
 
 import clickzetta
-import sqlparse
 import sqlalchemy
 import json
 import random
@@ -69,7 +68,7 @@ DEFAULT_INSTANCE_ID = 100
 class Client(object):
 
     def __init__(self, login_params: LoginParams = None, workspace: str = None, instance_name: str = None,
-                 vc_name: str = None, cz_url: str = None, base_url: str = None):
+                 vc_name: str = None, cz_url: str = None, base_url: str = None, schema: str = None):
         if cz_url is not None:
             self._parse_url(cz_url)
             self.token = self.log_in_cz(self.login_params)
@@ -88,6 +87,7 @@ class Client(object):
             self.session = None
             self.instance_id = 0
             self.username = login_params.username
+            self.schema = schema
 
     def _parse_url(self, url: string):
         url = sqlalchemy.make_url(url)
@@ -108,6 +108,8 @@ class Client(object):
                 self.vc_name = query.pop('virtualCluster')
         else:
             raise ValueError('url must have `virtualcluster` parameter.')
+        if 'schema' in query:
+            self.schema = query.pop('schema')
 
         self.login_params = LoginParams(self.username, pwd, self.instance_name)
 
@@ -127,31 +129,6 @@ class Client(object):
         except requests.exceptions.RequestException:
             raise
 
-    def get_table_infos_from_show_statement(self, sql: string):
-        format_sql = sqlparse.format(sql, reindent=True, keyword_case='upper')
-        stmt = sqlparse.parse(format_sql)[0]
-        columns = []
-        types = []
-
-        for token in stmt.tokens:
-            if not isinstance(token, sqlparse.sql.Parenthesis):
-                continue
-            else:
-                for sub_token in token.tokens:
-                    if isinstance(sub_token, sqlparse.sql.Identifier) and not sub_token.value.startswith('NOVALID'):
-                        columns.append(sub_token.value.upper())
-                    elif isinstance(sub_token, sqlparse.sql.Token) and sub_token.ttype is not None and len(
-                            sub_token.ttype) == 2 and sub_token.ttype[
-                        1] == 'Builtin':
-                        types.append(sub_token.value.replace('\\', '').upper())
-                    elif isinstance(sub_token, sqlparse.sql.Function) and not sub_token.value.startswith(
-                            "UNIQUE") and not sub_token.value.startswith("KEY"):
-                        types.append(sub_token.value.replace('\\', '').upper())
-                    else:
-                        continue
-                break
-        return {'columns': columns, 'types': types}
-
     def create_bulkload_stream(self, schema_name: string, table_name: string,
                                options: BulkLoadOptions) -> BulkLoadMetaData:
         create_bulk_load_request = ingestion_pb2.CreateBulkLoadStreamRequest()
@@ -333,9 +310,7 @@ class Client(object):
         table = Table(self.workspace, '', self.instance_name, self.vc_name)
         vc = table.vc_name
         job_type = JobType.SQL_JOB
-        unique_id = str(datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'))
-        format_unique_id = unique_id.replace('-', '').replace(':', '').replace('.', '').replace(' ', '') \
-                           + str(random.randint(10000, 99999))
+        format_unique_id = self._format_job_id()
         job_id = JobID(format_unique_id, table.workspace, DEFAULT_INSTANCE_ID)
         logging.info(
             "python sqlalchemy job id:" + format_unique_id)
@@ -349,6 +324,8 @@ class Client(object):
         schema_name = ''
         if schema is not None:
             schema_name = schema
+        elif self.schema is not None:
+            schema_name = self.schema
         sql_job = SQLJob(sql_config, table.workspace, schema_name, [])
         sql_job.query.append(query)
         job_timeout_ms = 3000
@@ -363,9 +340,11 @@ class Client(object):
         HEADERS['X-ClickZetta-Token'] = token
         try:
             api_response = self.session.post(self.base_url + path, data=data, headers=HEADERS)
+            api_response.encoding = 'utf-8'
             result = api_response.text
             result_dict = json.loads(result)
-            query_result = QueryResult(result_dict, 'show')
+            get_job_result_dict = self.wait_job_finished(result_dict, job_id, HEADERS)
+            query_result = QueryResult(get_job_result_dict, 'show')
             return query_result
 
         except requests.exceptions.RequestException:
@@ -376,9 +355,7 @@ class Client(object):
         table = Table(self.workspace, '', self.instance_name, self.vc_name)
         vc = table.vc_name
         job_type = JobType.SQL_JOB
-        unique_id = str(datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'))
-        format_unique_id = unique_id.replace('-', '').replace(':', '').replace('.', '').replace(' ', '') \
-                           + str(random.randint(10000, 99999))
+        format_unique_id = self._format_job_id()
         job_id = JobID(format_unique_id, table.workspace, DEFAULT_INSTANCE_ID)
         logging.info(
             "python sqlalchemy job id:" + format_unique_id)
@@ -392,6 +369,8 @@ class Client(object):
         schema_name = ''
         if schema is not None:
             schema_name = schema
+        elif self.schema is not None:
+            schema_name = self.schema
         sql_job = SQLJob(sql_config, table.workspace, schema_name, [])
         sql_job.query.append(query)
         job_timeout_ms = 3000
@@ -406,9 +385,11 @@ class Client(object):
         HEADERS['X-ClickZetta-Token'] = token
         try:
             api_response = self.session.post(self.base_url + path, data=data, headers=HEADERS)
+            api_response.encoding = 'utf-8'
             result = api_response.text
             result_dict = json.loads(result)
-            query_result = QueryResult(result_dict, 'desc')
+            get_job_result_dict = self.wait_job_finished(result_dict, job_id, HEADERS)
+            query_result = QueryResult(get_job_result_dict, 'desc')
             return query_result
 
         except requests.exceptions.RequestException:
@@ -419,9 +400,7 @@ class Client(object):
         table = Table(self.workspace, '', self.instance_name, self.vc_name)
         vc = table.vc_name
         job_type = JobType.SQL_JOB
-        unique_id = str(datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'))
-        format_unique_id = unique_id.replace('-', '').replace(':', '').replace('.', '').replace(' ', '') \
-                           + str(random.randint(10000, 99999))
+        format_unique_id = self._format_job_id()
         job_id = JobID(format_unique_id, table.workspace, DEFAULT_INSTANCE_ID)
         logging.info(
             "python sqlalchemy job id:" + format_unique_id)
@@ -432,7 +411,10 @@ class Client(object):
         query = query_sql
         job_config = {}
         sql_config = SQLJobConfig(0, "0", "0", {'cz.sql.adhoc.result.type': 'embedded'})
-        sql_job = SQLJob(sql_config, table.workspace, '', [])
+        schema_name = ''
+        if self.schema is not None:
+            schema_name = self.schema
+        sql_job = SQLJob(sql_config, table.workspace, schema_name, [])
         sql_job.query.append(query)
         job_timeout_ms = 3000
         user_agent = ""
@@ -446,9 +428,11 @@ class Client(object):
         HEADERS['X-ClickZetta-Token'] = token
         try:
             api_response = self.session.post(self.base_url + path, data=data, headers=HEADERS)
+            api_response.encoding = 'utf-8'
             result = api_response.text
             result_dict = json.loads(result)
-            query_result = QueryResult(result_dict, 'drop')
+            get_job_result_dict = self.wait_job_finished(result_dict, job_id, HEADERS)
+            query_result = QueryResult(get_job_result_dict, 'drop')
             return query_result
         except requests.exceptions.RequestException:
             raise
@@ -458,9 +442,7 @@ class Client(object):
         table = Table(self.workspace, '', self.instance_name, self.vc_name)
         vc = table.vc_name
         job_type = JobType.SQL_JOB
-        unique_id = str(datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'))
-        format_unique_id = unique_id.replace('-', '').replace(':', '').replace('.', '').replace(' ', '') \
-                           + str(random.randint(10000, 99999))
+        format_unique_id = self._format_job_id()
         job_id = JobID(format_unique_id, table.workspace, DEFAULT_INSTANCE_ID)
         logging.info(
             "python sqlalchemy job id:" + format_unique_id)
@@ -470,7 +452,10 @@ class Client(object):
         timeout = 100
         job_config = {}
         sql_config = SQLJobConfig(0, "0", "0", {'cz.sql.adhoc.result.type': 'embedded'})
-        sql_job = SQLJob(sql_config, table.workspace, '', [])
+        schema_name = ''
+        if self.schema is not None:
+            schema_name = self.schema
+        sql_job = SQLJob(sql_config, table.workspace, schema_name, [])
         sql_job.query.append(alter_sql)
         job_timeout_ms = 3000
         user_agent = ""
@@ -484,9 +469,11 @@ class Client(object):
         HEADERS['X-ClickZetta-Token'] = token
         try:
             api_response = self.session.post(self.base_url + path, data=data, headers=HEADERS)
+            api_response.encoding = 'utf-8'
             result = api_response.text
             result_dict = json.loads(result)
-            query_result = QueryResult(result_dict, 'alter')
+            get_job_result_dict = self.wait_job_finished(result_dict, job_id, HEADERS)
+            query_result = QueryResult(get_job_result_dict, 'alter')
             return query_result
         except requests.exceptions.RequestException:
             raise
@@ -496,9 +483,7 @@ class Client(object):
         table = Table(self.workspace, '', self.instance_name, self.vc_name)
         vc = table.vc_name
         job_type = JobType.SQL_JOB
-        unique_id = str(datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'))
-        format_unique_id = unique_id.replace('-', '').replace(':', '').replace('.', '').replace(' ', '') \
-                           + str(random.randint(10000, 99999))
+        format_unique_id = self._format_job_id()
         job_id = JobID(format_unique_id, table.workspace, DEFAULT_INSTANCE_ID)
         logging.info(
             "python sqlalchemy job id:" + format_unique_id)
@@ -509,7 +494,10 @@ class Client(object):
         job_config = {}
         query = query_sql
         sql_config = SQLJobConfig(0, "0", "0", {'cz.sql.adhoc.result.type': 'embedded'})
-        sql_job = SQLJob(sql_config, table.workspace, '', [])
+        schema_name = ''
+        if self.schema is not None:
+            schema_name = self.schema
+        sql_job = SQLJob(sql_config, table.workspace, schema_name, [])
         sql_job.query.append(query)
         job_timeout_ms = 3000
         user_agent = ""
@@ -523,9 +511,11 @@ class Client(object):
         HEADERS['X-ClickZetta-Token'] = token
         try:
             api_response = self.session.post(self.base_url + path, data=data, headers=HEADERS)
+            api_response.encoding = 'utf-8'
             result = api_response.text
             result_dict = json.loads(result)
-            query_result = QueryResult(result_dict, 'truncate')
+            get_job_result_dict = self.wait_job_finished(result_dict, job_id, HEADERS)
+            query_result = QueryResult(get_job_result_dict, 'truncate')
             return query_result
         except requests.exceptions.RequestException:
             raise
@@ -535,9 +525,7 @@ class Client(object):
         table = Table(self.workspace, '', self.instance_name, self.vc_name)
         vc = table.vc_name
         job_type = JobType.SQL_JOB
-        unique_id = str(datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'))
-        format_unique_id = unique_id.replace('-', '').replace(':', '').replace('.', '').replace(' ', '') \
-                           + str(random.randint(10000, 99999))
+        format_unique_id = self._format_job_id()
         job_id = JobID(format_unique_id, table.workspace, DEFAULT_INSTANCE_ID)
         logging.info(
             "python sqlalchemy job id:" + format_unique_id)
@@ -547,7 +535,10 @@ class Client(object):
         timeout = 100
         job_config = {}
         sql_config = SQLJobConfig(0, "0", "0", {'cz.sql.adhoc.result.type': 'embedded'})
-        sql_job = SQLJob(sql_config, table.workspace, '', [])
+        schema_name = ''
+        if self.schema is not None:
+            schema_name = self.schema
+        sql_job = SQLJob(sql_config, table.workspace, schema_name, [])
         sql_job.query.append(create_sql)
         job_timeout_ms = 3000
         user_agent = ""
@@ -561,9 +552,11 @@ class Client(object):
         HEADERS['X-ClickZetta-Token'] = token
         try:
             api_response = self.session.post(self.base_url + path, data=data, headers=HEADERS)
+            api_response.encoding = 'utf-8'
             result = api_response.text
             result_dict = json.loads(result)
-            query_result = QueryResult(result_dict, 'create')
+            get_job_result_dict = self.wait_job_finished(result_dict, job_id, HEADERS)
+            query_result = QueryResult(get_job_result_dict, 'create')
             return query_result
         except requests.exceptions.RequestException:
             raise
@@ -575,9 +568,7 @@ class Client(object):
         table = Table(self.workspace, '', self.instance_name, self.vc_name)
         vc = table.vc_name
         job_type = JobType.SQL_JOB
-        unique_id = str(datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'))
-        format_unique_id = unique_id.replace('-', '').replace(':', '').replace('.', '').replace(' ', '') \
-                           + str(random.randint(10000, 99999))
+        format_unique_id = self._format_job_id()
         job_id = JobID(format_unique_id, table.workspace, DEFAULT_INSTANCE_ID)
         logging.info(
             "python sqlalchemy job id:" + format_unique_id)
@@ -592,7 +583,10 @@ class Client(object):
                 for key in parameters['hints']:
                     sql_job_set_prop[key] = parameters['hints'][key]
         sql_config = SQLJobConfig(0, "0", "0", sql_job_set_prop)
-        sql_job = SQLJob(sql_config, table.workspace, '', [])
+        schema_name = ''
+        if self.schema is not None:
+            schema_name = self.schema
+        sql_job = SQLJob(sql_config, table.workspace, schema_name, [])
         sql_job.query.append(select_sql)
         job_timeout_ms = 3000
         user_agent = ""
@@ -622,9 +616,7 @@ class Client(object):
         table = Table(self.workspace, '', self.instance_name, self.vc_name)
         vc = table.vc_name
         job_type = JobType.SQL_JOB
-        unique_id = str(datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'))
-        format_unique_id = unique_id.replace('-', '').replace(':', '').replace('.', '').replace(' ', '') \
-                           + str(random.randint(10000, 99999))
+        format_unique_id = self._format_job_id()
         job_id = JobID(format_unique_id, table.workspace, DEFAULT_INSTANCE_ID)
         logging.info(
             "python sqlalchemy job id:" + format_unique_id)
@@ -634,7 +626,10 @@ class Client(object):
         timeout = 10
         job_config = {}
         sql_config = SQLJobConfig(0, "0", "0", {'cz.sql.adhoc.result.type': 'embedded'})
-        sql_job = SQLJob(sql_config, table.workspace, '', [])
+        schema_name = ''
+        if self.schema is not None:
+            schema_name = self.schema
+        sql_job = SQLJob(sql_config, table.workspace, schema_name, [])
         sql_job.query.append(insert_sql)
         job_timeout_ms = 3000
         user_agent = ""
@@ -661,9 +656,7 @@ class Client(object):
         table = Table(self.workspace, '', self.instance_name, self.vc_name)
         vc = table.vc_name
         job_type = JobType.SQL_JOB
-        unique_id = str(datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'))
-        format_unique_id = unique_id.replace('-', '').replace(':', '').replace('.', '').replace(' ', '') \
-                           + str(random.randint(10000, 99999))
+        format_unique_id = self._format_job_id()
         job_id = JobID(format_unique_id, table.workspace, DEFAULT_INSTANCE_ID)
         logging.info(
             "python sqlalchemy job id:" + format_unique_id)
@@ -673,7 +666,10 @@ class Client(object):
         timeout = 100
         job_config = {}
         sql_config = SQLJobConfig(0, "0", "0", {'cz.sql.adhoc.result.type': 'embedded'})
-        sql_job = SQLJob(sql_config, table.workspace, '', [])
+        schema_name = ''
+        if self.schema is not None:
+            schema_name = self.schema
+        sql_job = SQLJob(sql_config, table.workspace, schema_name, [])
         sql_job.query.append(update_sql)
         job_timeout_ms = 3000
         user_agent = ""
@@ -687,9 +683,53 @@ class Client(object):
         HEADERS['X-ClickZetta-Token'] = token
         try:
             api_response = self.session.post(self.base_url + path, data=data, headers=HEADERS)
+            api_response.encoding = 'utf-8'
             result = api_response.text
             result_dict = json.loads(result)
-            query_result = QueryResult(result_dict, 'update')
+            get_job_result_dict = self.wait_job_finished(result_dict, job_id, HEADERS)
+            query_result = QueryResult(get_job_result_dict, 'update')
+            return query_result
+        except requests.exceptions.RequestException:
+            raise
+
+    def execute_other_sql(self, token: str, sql: str) -> QueryResult:
+        path = "/lh/submitJob"
+        table = Table(self.workspace, '', self.instance_name, self.vc_name)
+        vc = table.vc_name
+        job_type = JobType.SQL_JOB
+        format_unique_id = self._format_job_id()
+        job_id = JobID(format_unique_id, table.workspace, DEFAULT_INSTANCE_ID)
+        logging.info(
+            "python sqlalchemy job id:" + format_unique_id)
+        job_name = "OTHER_SQL"
+        user_id = 0
+        reqeust_mode = JobRequestMode.HYBRID
+        timeout = 100
+        job_config = {}
+        sql_config = SQLJobConfig(0, "0", "0",
+                                  {'cz.sql.adhoc.result.type': 'embedded'})
+        schema_name = ''
+        if self.schema is not None:
+            schema_name = self.schema
+        sql_job = SQLJob(sql_config, table.workspace, schema_name, [])
+        sql_job.query.append(sql)
+        job_timeout_ms = 3000
+        user_agent = ""
+
+        job_desc = JobDesc(vc, job_type, job_id, job_name, user_id, reqeust_mode, timeout, job_config, sql_job,
+                           job_timeout_ms,
+                           user_agent, 0)
+        job_request = JobRequest(job_desc)
+        data = json.dumps(job_request.to_api_repr())
+        HEADERS['instanceName'] = table.instance_name
+        HEADERS['X-ClickZetta-Token'] = token
+        try:
+            api_response = self.session.post(self.base_url + path, data=data, headers=HEADERS)
+            api_response.encoding = 'utf-8'
+            result = api_response.text
+            result_dict = json.loads(result)
+            get_job_result_dict = self.wait_job_finished(result_dict, job_id, HEADERS)
+            query_result = QueryResult(get_job_result_dict, 'other')
             return query_result
         except requests.exceptions.RequestException:
             raise
@@ -698,26 +738,51 @@ class Client(object):
         print("client is closed")
 
     def wait_job_finished(self, result_dict, job_id: JobID, headers):
-        status = result_dict['status']['state']
-        if status == "QUEUEING" or status == "SETUP" or status == "RUNNING":
-            account = clickzetta.enums.ClickZettaAccount(0)
-            get_job_result_request = clickzetta.enums.GetJobResultRequest(account, job_id, 0, '')
-            api_request = clickzetta.enums.APIJobRequest(get_job_result_request, '')
-            path = '/lh/getJob'
-            data = json.dumps(api_request.to_api_repr())
-            while True:
-                get_job_result_response = self.session.post(self.base_url + path, data=data, headers=headers)
-                result = get_job_result_response.text
-                get_job_result_dict = json.loads(result)
-                if get_job_result_dict['status']['state'] == 'SUCCEED' or get_job_result_dict['status']['state'] \
-                        == 'FAILED':
-                    logging.info("Get async sql job:" + str(job_id.id) + " result successfully.")
-                    return get_job_result_dict
-                else:
-                    logging.info("Get async sql job:" + str(job_id.id) + " result pending, retry after 10 seconds.")
-                    time.sleep(10)
-        elif status == "SUCCEED" or status == 'FAILED':
-            return result_dict
+        if 'status' in result_dict:
+            if 'state' in result_dict['status']:
+                status = result_dict['status']['state']
+                if status == "QUEUEING" or status == "SETUP" or status == "RUNNING":
+                    account = clickzetta.enums.ClickZettaAccount(0)
+                    get_job_result_request = clickzetta.enums.GetJobResultRequest(account, job_id, 0, '')
+                    api_request = clickzetta.enums.APIJobRequest(get_job_result_request, '')
+                    path = '/lh/getJob'
+                    data = json.dumps(api_request.to_api_repr())
+                    while True:
+                        get_job_result_response = self.session.post(self.base_url + path, data=data, headers=headers)
+                        result = get_job_result_response.text
+                        get_job_result_dict = json.loads(result)
+                        if 'status' in get_job_result_dict:
+                            if 'state' in get_job_result_dict['status']:
+                                if get_job_result_dict['status']['state'] == 'SUCCEED' or \
+                                        get_job_result_dict['status']['state'] == 'FAILED':
+                                    logging.info(
+                                        "Get async sql job:" + str(job_id.id) + " result successfully.")
+                                    return get_job_result_dict
+                                else:
+                                    logging.info("Get async sql job:" + str(
+                                        job_id.id) + " result pending, retry after 10 seconds.")
+                                    time.sleep(10)
+                            else:
+                                raise Exception(
+                                    "Get async sql job:" + str(
+                                        job_id.id) + " result failed, http response job state is not exist.")
+                        else:
+                            raise Exception(
+                                "Get async sql job:" + str(
+                                    job_id.id) + " result failed, http response job status is not exist.")
+
+                elif status == "SUCCEED" or status == 'FAILED':
+                    return result_dict
+            else:
+                raise Exception("Execute sql job:" + str(job_id.id) + " failed, http response job state is not exist.")
+        else:
+            raise Exception("Execute sql job:" + str(job_id.id) + " failed, http response job status is not exist.")
+
+    def _format_job_id(self):
+        unique_id = str(datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f'))
+        format_unique_id = unique_id.replace('-', '').replace(':', '').replace('.', '').replace(' ', '') \
+                           + str(random.randint(10000, 99999))
+        return format_unique_id
 
     def get_table_names(self, schema: str):
         query_result = self.show_table(self.token, 'show tables;', schema)
